// Include DTSource.h if you want to include all the headers.

#include "DTArguments.h"
#include "DTSaveError.h"

#include "DTDataFile.h"
#include "DTDoubleArray.h"
#include "DTDoubleArrayOperators.h"

// This was added to the automatically created main.cpp file
#include "DTSeriesNumberList.h"
#include "DTDoubleArrayRegion.h"
#include "DTProgress.h"

#include <math.h>

void Computation(DTDataFile in,string name,double dt,DTDataFile &outputFile);

int main(int argc,const char *argv[])
{
    DTSetArguments(argc,argv);

    DTDataFile inputFile("Input.dtbin",DTFile::ReadOnly);
    // Read in the input variables.
    DTDataFile in = DTDataFile("in.dtbin",DTFile::ReadOnly);
    string name = inputFile.ReadString("name");
    double dt = inputFile.ReadNumber("dt");


    // Create the data file in the main routine and pass it in.
    DTDataFile outputFileComputed("Output.dtbin",DTFile::NewReadWrite);

    // The computation.
    clock_t t_before = clock();
    Computation(in,name,dt,outputFileComputed);
    clock_t t_after = clock();
    double exec_time = double(t_after-t_before)/double(CLOCKS_PER_SEC);

    DTDataFile outputFile("ExecutionInformation",DTFile::NewReadWrite);

    // The execution time.
    outputFile.Save(exec_time,"ExecutionTime");
    outputFile.Save("Real Number","Seq_ExecutionTime");

    // The errors.
    DTSaveError(outputFile,"ExecutionErrors");
    outputFile.Save("StringList","Seq_ExecutionErrors");

    return 0;
}

void Computation(DTDataFile in,string name,double dt,DTDataFile &outputFile)
{
    // If the input name wasn't in.dtbin the input file would just be a DTFile object
    // and you can extract the FILE pointer from there.  Or change the main routine
    // above to open the file from the file name.
    
    DTDoubleArray array = in.ReadDoubleArray(name);
    int m = array.m();
    int n = array.n();
    
    DTProgress progress; // To let DataTank know how far you are in the conversion - optional
    
    // Create a convenience object to save a time series of numbers.  You can have any number of these
    DTSeriesNumberList columnList(outputFile,"Column");
    DTSeriesNumberList rowList(outputFile,"Row");
    
    DTDoubleArray entry;
    int i,j;
    for (j=0;j<n;j++) {
        progress.UpdatePercentage(j/double(n+m));
        
        entry = array(DTIndex::All,j); // Enabled by the DTDoubleArrayRegion header
        
        // This adds the entry to the data file.  It is saved immediately to the file
        columnList.Add(entry,j*dt);
    }

    for (i=0;i<m;i++) {
        progress.UpdatePercentage((n+i)/double(n+m));
        rowList.Add(array(i,DTIndex::All),i*dt);
    }
}
